import { Nullable } from "../utils";
export interface IMapAccess {
    nextKeySeed<T, K extends Deserialize>(seed: K): IteratorResult<T>;
    nextValueSeed<T, V extends Deserialize>(seed: V): IteratorResult<T>;
    nextEntrySeed<TK, TV, K extends Deserialize, V extends Deserialize>(kseed: K, vseed: V): IteratorResult<[TK, TV]>;
    nextKey<T>(): IteratorResult<T>;
    nextValue<V>(): IteratorResult<V>;
    nextEntry<K, V>(): IteratorResult<[K, V]>;
    sizeHint?(): Nullable<number>;
}
export declare abstract class MapAccess {
    abstract nextKeySeed<T, K extends Deserialize>(seed: K): IteratorResult<T>;
    abstract nextValueSeed<T, V extends Deserialize>(seed: V): IteratorResult<T>;
    nextEntrySeed<TK, TV, K extends Deserialize, V extends Deserialize>(kseed: K, vseed: V): IteratorResult<[TK, TV]>;
    abstract nextKey<T>(): IteratorResult<T>;
    abstract nextValue<V>(): IteratorResult<V>;
    nextEntry<K, V>(): IteratorResult<[K, V]>;
}
export interface IIterableAccess {
    nextElement<T>(): IteratorResult<T>;
    sizeHint?(): Nullable<number>;
}
export declare abstract class IterableAccess implements IIterableAccess {
    abstract nextElement<T>(): IteratorResult<T>;
}
export interface IVisitor<T> {
    visitBoolean(value: boolean): T;
    visitNumber(value: number): T;
    visitBigInt(value: bigint): T;
    visitString(value: string): T;
    visitSymbol(value: symbol): T;
    visitNull(): T;
    visitObject(access: IMapAccess): T;
    visitIterable(access: IIterableAccess): T;
}
export interface IDeserializer {
    deserializeAny<T, V extends IVisitor<T>>(visitor: V): T;
    deserializeBoolean<T, V extends IVisitor<T>>(visitor: V): T;
    deserializeNumber<T, V extends IVisitor<T>>(visitor: V): T;
    deserializeBigInt<T, V extends IVisitor<T>>(visitor: V): T;
    deserializeString<T, V extends IVisitor<T>>(visitor: V): T;
    deserializeSymbol<T, V extends IVisitor<T>>(visitor: V): T;
    deserializeNull<T, V extends IVisitor<T>>(visitor: V): T;
    deserializeObject<T, V extends IVisitor<T>>(visitor: V): T;
    deserializeIterable<T, V extends IVisitor<T>>(visitor: V): T;
    deserializeFunction<T, V extends IVisitor<T>>(visitor: V): T;
}
export interface Deserialize {
    <T>(deserializer: IDeserializer): T;
}
