import { IDeserializer, IIterableAccess, IMapAccess, IVisitor } from './interface'

export class GenericSeed<T> {
  readonly visitor: IVisitor<T>

  constructor(visitor: IVisitor<T> = new GenericVisitor()) {
    this.visitor = visitor
  }

  static deserialize<T, D extends IDeserializer>(deserializer: D, visitor: IVisitor<T> = new GenericVisitor()): T {
    return deserializer.deserializeAny(visitor)
  }

  deserialize<D extends IDeserializer>(deserializer: D): T {
    return GenericSeed.deserialize(deserializer, this.visitor)
  }
}

export class GenericVisitor<T> implements IVisitor<T> {
  visitBoolean(value: boolean): T {
    return value as T
  }

  visitNumber(value: number): T {
    return value as T
  }

  visitBigInt(value: bigint): T {
    return value as T
  }

  visitString(value: string): T {
    return value as T
  }

  visitSymbol(value: symbol): T {
    return value as T
  }

  visitNull(): T {
    return null as T
  }

  visitObject(access: IMapAccess): T {
    const result = []
    let entry

    while ((entry = access.nextEntry()) && !entry.done) {
      result.push(entry.value)
    }

    return Object.fromEntries(result)
  }

  visitIterable(access: IIterableAccess): T {
    const result = []
    let element

    while ((element = access.nextElement())) {
      result.push(element)
    }

    return result as T
  }
}



