export type Nullable<T> = T | undefined

export interface Morphism<T = any, U = any> {
  (value: T): U
}

export type Primitive = string | number | boolean | symbol | bigint | null | undefined

export interface ToString {
  toString(): string
}

export function staticImplements<T>() {
  return <U extends T>(constructor: U) => { constructor }
}

export function isPlainObject(value: any): boolean {
  return Object.getPrototypeOf(value) === Object.prototype
}

export function isFunction(value: any): value is Function {
  return value != null && typeof value === 'function'
}

export function isIterable(value: any): value is Iterable<any> {
  return isFunction(value[Symbol.iterator])
}

export function isString(value: any): value is string {
  return typeof value === 'string'
}

export function isNumber(value: any): value is number {
  return !isNaN(value)
}

export type Constructor<T = any> = new (...args: any[]) => T

export class IterResult {
  static Next<T>(value: T): IteratorResult<T> {
    return { done: false, value }
  }

  static Done<T>(): IteratorResult<T> {
    return { done: true, value: undefined }
  }
}

